<?php

class ControllerExtensionPaymentCkassa extends Controller {

    const API_VERSION = 1.0;
    const TEST_API_URL = 'https://demo-api2.ckassa.ru/api-shop/rs/opencart/do-pay/anonymous';
    const API_URL = 'https://api2.ckassa.ru/api-shop/rs/opencart/do-pay/anonymous';
    const TEST_SHOP_KEY = '183bdddf-6a50-4942-bc93-a0c4f06af759';
    const TEST_SEC_KEY = '6746a6d9-41d2-448d-9c64-8f38d98af368';

    public function index() {

        $data['continue'] = $this->url->link('checkout/success');

        return $this->load->view('extension/payment/ckassa', $data);
    }

    function prepareConfigParameters() {

        $config = array(
            'ckassa_url' => self::API_URL,
            'ckassa_is_test' => $this->config->get('ckassa_is_test') == 1,
            'ckassa_tax' => $this->config->get('ckassa_tax'),
            'ckassa_payment_method' => $this->config->get('ckassa_payment_method'),
            'ckassa_payment_object' => $this->config->get('ckassa_payment_object'),
            'ckassa_tax_delivery' => $this->config->get('ckassa_tax_delivery'),
            'ckassa_shop_key' => $this->config->get('ckassa_shop_key'),
            'ckassa_country' => $this->config->get('ckassa_country'),
            'ckassa_taxation' => $this->config->get('ckassa_taxation'),
            'ckassa_fail_url' => $this->config->get('ckassa_fail_url'),
            'ckassa_success_url' => $this->config->get('ckassa_success_url'),
            'ckassa_cb_url' => $this->config->get('ckassa_cb_url'),
            'ckassa_secret' => $this->config->get('ckassa_secret'),
            'ckassa_order_status_after_payment' => $this->config->get('ckassa_order_status_after_payment'),
            'ckassa_order_status_before_payment' => $this->config->get('ckassa_order_status_before_payment'),
            'ckassa_currency' => 'RUB',
            'ckassa_lang' => 'RU'
        );

        if ($config['ckassa_is_test']) {

            $config['ckassa_url'] = self::TEST_API_URL;
            $config['ckassa_shop_key'] = self::TEST_SHOP_KEY;
            $config['ckassa_secret'] = self::TEST_SEC_KEY;
            $config['ckassa_invunic'] = 'NONE';
        }

        return $config;
    }

    /**
     * 
     * @staticvar array $config
     * @param string $name
     * @return mixed
     */
    function getConfigParameter($name) {

        static $config = array();
        if (empty($config)) {

            $config = $this->prepareConfigParameters();
        }

        return $config[$name] ? $config[$name] : null;
    }

    public function confirm() {
        $this->language->load('payment/ckassa');
        $this->load->model('checkout/order');
        $this->load->model('account/order');
        $this->load->model('catalog/product');

        $order = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $products = $this->model_account_order->getOrderProducts($this->session->data['order_id']);

        $orderItems = [];
        $config_fee = [
            'handling' => [
                'status' => $this->config->get('total_handling_status'), #статус
                'fee' => $this->config->get('total_handling_fee'), #добавочная плата за превышение лимита
                'total' => $this->config->get('total_handling_total') #от какой суммы добавляется
            ],
            'low_order' => [
                'status' => $this->config->get('total_low_order_fee_status'), #статус
                'fee' => $this->config->get('total_low_order_fee_fee'), #добавочная плата за минимум
                'total' => $this->config->get('total_low_order_fee_total') #до какой суммы добавляется
            ]
            ];

        #$orderAmount = $this->toCoins($order['total']);
        $orderAmount = 0;

        $parameters = [
            'shop' => $this->getConfigParameter('ckassa_shop_key'),
            'currency' => $this->getConfigParameter('ckassa_currency'),
            'language' => $this->getConfigParameter('ckassa_lang'),
            'country' => $this->getConfigParameter('ckassa_country'),
            'orderId' => $this->session->data['order_id'],
            'cbUrl' => $this->getConfigParameter('ckassa_cb_url'),
            'successUrl' => $this->getConfigParameter('ckassa_success_url'),
            'failUrl' => $this->getConfigParameter('ckassa_fail_url'),
            #'amount' => $orderAmount,
            'desc' => '',
            'email' => $order['email'],
            'phone' => $order['telephone'],
            'fio' => $order['firstname'] . ' ' . $order['lastname'],
            #'signature' => $this->getRequestSignature($orderAmount, $order['order_id']),
        ];
        $sum_orderItems = 0;
        if ($this->getConfigParameter('ckassa_country') == 'RU') {
            foreach ($products as $k => $product) {
                $productInfo = $this->model_catalog_product->getProduct($product['product_id']);
                // $price = $this->formatAmount((float) $product['price'] + (float) $product['tax']);
                $price = $this->formatAmount((float) $product['price']);
                $sum = $this->formatAmount($price * $product['quantity']);
                $sum_orderItems += $sum;
                $orderAmount += $sum;
                $orderItems[] = array(
                    'num' => $k,
                    'name' => $productInfo['name'],
                    'quantity' => $product['quantity'],
                    'price' => $price,
                    'sum' => $sum,
                    'tax' => $this->getConfigParameter('ckassa_tax'),
                    'payment_method' => $this->getConfigParameter('ckassa_payment_method'),
                    'payment_object' => $this->getConfigParameter('ckassa_payment_object')
                );
            }

            if (isset($this->session->data['shipping_method']) && isset($this->session->data['shipping_method']['cost']) && $this->session->data['shipping_method']['cost'] > 0) {

                $cost = $this->session->data['shipping_method']['cost'];
                // if ($this->session->data['shipping_method']['tax_class_id'] > 0) {
                //     $taxRates = $this->tax->getRates($this->session->data['shipping_method']['cost'], $this->session->data['shipping_method']['tax_class_id']);
                //     if (!empty($taxRates)) {
                //         foreach ($taxRates as $taxRate) {
                //             $cost += $taxRate['amount'];
                //         }
                //     }
                // }
                $orderAmount += $this->formatAmount($cost);
                $orderItems[] = [
                    'num' => count($orderItems),
                    'name' => $this->language->get('text_delivery'),
                    'quantity' => 1,
                    'price' => $this->formatAmount($cost),
                    'sum' => $this->formatAmount($cost),
                    'tax' => $this->getConfigParameter('ckassa_tax_delivery'),
                    'payment_method' => 'full_prepayment',
                    'payment_object' => 'service'
                ];
            }
            if ($config_fee['handling']['status'] && $config_fee['handling']['total'] < $sum_orderItems) {
                $orderAmount += $this->formatAmount($config_fee['handling']['fee']);
                $orderItems[] = [
                    'num' => count($orderItems),
                    'name' => 'Плата за обработку заказа',
                    'quantity' => 1,
                    'price' => $this->formatAmount($config_fee['handling']['fee']),
                    'sum' => $this->formatAmount($config_fee['handling']['fee']),
                    'tax' => $this->getConfigParameter('ckassa_tax_delivery'),
                    'payment_method' => 'full_prepayment',
                    'payment_object' => 'service'
                ];
            }
            if ($config_fee['low_order']['status'] && $config_fee['low_order']['total'] > $sum_orderItems) {
                $orderAmount += $this->formatAmount($config_fee['low_order']['fee']);
                $orderItems[] = [
                    'num' => count($orderItems),
                    'name' => 'Доп. плата при минимальном заказе',
                    'quantity' => 1,
                    'price' => $this->formatAmount($config_fee['low_order']['fee']),
                    'sum' => $this->formatAmount($config_fee['low_order']['fee']),
                    'tax' => $this->getConfigParameter('ckassa_tax_delivery'),
                    'payment_method' => 'full_prepayment',
                    'payment_object' => 'service'
                ];
            }
            $parameters['Receipt'] = base64_encode(json_encode([
                'items' => $orderItems,
                'taxation' => $this->getConfigParameter('ckassa_taxation'),
                            ], JSON_UNESCAPED_UNICODE));
        } else {
            foreach ($products as $k => $product) {
                $price = $this->formatAmount((float) $product['price'] + (float) $product['tax']);
                $sum = $this->formatAmount($price * $product['quantity']);
                $orderAmount += $sum;
            }
            if ($config_fee['handling']['status'] && $config_fee['handling']['total'] < $sum_orderItems) {
                $orderAmount += $this->formatAmount($config_fee['handling']['fee']);
            }
            if ($config_fee['low_order']['status'] && $config_fee['low_order']['total'] > $sum_orderItems) {
                $orderAmount += $this->formatAmount($config_fee['low_order']['fee']);
            }
            if (isset($this->session->data['shipping_method']) && isset($this->session->data['shipping_method']['cost']) && $this->session->data['shipping_method']['cost'] > 0) {
                $cost = $this->session->data['shipping_method']['cost'];
                if ($this->session->data['shipping_method']['tax_class_id'] > 0) {
                    $taxRates = $this->tax->getRates($this->session->data['shipping_method']['cost'], $this->session->data['shipping_method']['tax_class_id']);
                    if (!empty($taxRates)) {
                        foreach ($taxRates as $taxRate) {
                            $cost += $taxRate['amount'];
                        }
                    }
                }
                $orderAmount += $this->formatAmount($cost);
            }
        }
        $parameters['amount'] = $this->toCoins($orderAmount);
        $parameters['signature'] = $this->getRequestSignature($this->toCoins($orderAmount), $order['order_id']);


        if ($this->getConfigParameter('ckassa_is_test')) {
            $parameters['InvUnic'] = $this->getConfigParameter('ckassa_invunic');
        }

        $ctp_url = $this->getConfigParameter('ckassa_url');

        $curl = curl_init($ctp_url);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            'Content-Type:  application/x-www-form-urlencoded',
        ));
        curl_setopt($curl, CURLOPT_FORBID_REUSE, 1);
        curl_setopt($curl, CURLOPT_FRESH_CONNECT, 1);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($parameters));

        $responseContent = curl_exec($curl);
        curl_close($curl);

        if ($responseContent) {

            $responseData = json_decode($responseContent, true);

            if ($responseData && isset($responseData['payUrl'])) {

                $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->getConfigParameter('ckassa_order_status_before_payment'));

                $data = ['payUrl' => $responseData['payUrl']];
            } elseif ($responseData && $responseData['result']) {

                $data = ['errorMessage' => $responseData['result']['message']];
            }
        } else {
            $data = ['errorMessage' => 'System request error'];
        }

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($data));
    }

    /**
     * @param int $amount
     * @param int|string $orderId
     * @return string
     */
    function getRequestSignature($amount, $orderId) {

        return $this->createSignature([$orderId, $amount]);
    }

    /**
     * @param array $data
     * @return string
     */
    function createSignature(array $data) {
        $secret = $this->getConfigParameter('ckassa_secret');
        array_unshift($data, $this->getConfigParameter('ckassa_shop_key'));
        $data[] = $secret;
        $str = implode("|", $data);

        return base64_encode(strtoupper(hash_hmac('sha256', $str, $secret)));
    }

    /**
     * 
     * @param array $order
     * @param array $data
     * @return bool
     */
    function checkSignature(array $order, array $data) {
        return $this->createSignature([
                    $order['order_id'],
                    $data['regPayNum'],
                    $this->toCoins($order['total']),
                    $data['result']
                ]) == $data['signature'];
    }

    /**
     * 
     * @param int|float|string $amount
     * @return string
     */
    function formatAmount($amount) {
        return number_format((float) $amount, 2, '.', '');
    }

    /**
     * 
     * @param int|float|string $amount
     * @return string
     */
    function toCoins($amount) {
        return (string) round($this->formatAmount($amount) * 100);
    }

    public function callback() {

        $requestData = [
            'regPayNum' => trim($this->request->post['regPayNum']),
            'result' => trim($this->request->post['result']),
            'orderId' => trim($this->request->post['orderId']),
            'amount' => trim($this->request->post['amount']),
            'shop' => trim($this->request->post['shop']),
            'signature' => trim($this->request->post['signature']),
            'errorMsg' => htmlspecialchars(trim($this->request->post['errorMsg']))
        ];

        $this->load->model('checkout/order');
        $this->load->model('setting/setting');
        $this->language->load('extension/payment/ckassa');

        $order = $this->model_checkout_order->getOrder($requestData['orderId']);

        if (!$order || $order['order_status_id'] == $this->getConfigParameter('ckassa_order_status_after_payment')) {
            $this->log->write($this->language->get('payment_repeat_error'));
            die('fail');
        }

        if (
                $order && $this->toCoins($order['total']) == $requestData['amount'] &&
                isset($requestData['signature']) &&
                $this->checkSignature($order, $requestData) &&
                $requestData['shop'] == $this->getConfigParameter('ckassa_shop_key')
        ) {

            $message = "";
            switch ($requestData['result']) {
                case "success":

                    $message = str_replace(["#order_id", "#paynum"], [$order['order_id'], $requestData['regPayNum']], $this->language->get('payment_success'));
                    $this->model_checkout_order->addOrderHistory($order['order_id'], $this->getConfigParameter('ckassa_order_status_after_payment'), $message, true);
                    break;

                case "reject":

                    $message = str_replace(["#reason", "#order_id"], [@$requestData['errorMsg'] ?: "Unknown reason", $order['order_id']], $this->language->get('payment_reject'));
                    $this->log->write($message);
                    break;
            }

            echo "success";
        } else {
            $this->log->write($this->language->get('payment_bad_parameters'));
            echo "fail";
        }
        die;
    }

}

?>
